@@ -14,15 +14,19 @@
 #    under the License.
 
 import argparse
+import ConfigParser
 import logging
 import sys
 import os
-import ConfigParser
+
 import redmine
-from redmine.exceptions import *
+import tempfile
+from redmine import exceptions as rm_exc
+from subprocess import call
+
 
 # Setup the basic logging objects
-log = logging.getLogger('rore')
+LOG = logging.getLogger('rore')
 
 
 def get_user(rmine, userdata):
@@ -60,10 +64,11 @@ def print_issue(rmine, issue, verbose=False, oneline=False):
     print('type:        %s' % issue.tracker.name)
     try:
         print('assigned to: %s' % issue.assigned_to.name)
-    except ResourceAttrError:
+    except rm_exc.ResourceAttrError:
         print('assigned to: UNASSIGNED')
     print('project:     %s' % issue.project.name)
     print('status:      %s' % issue.status.name)
+    print('priority:    %s' % issue.priority.name)
     print('completion:  %s' % issue.done_ratio)
     if verbose:
         # Here is where we should enumerate all the possible fields
@@ -71,11 +76,11 @@ def print_issue(rmine, issue, verbose=False, oneline=False):
         print('start date:  %s' % issue.start_date)
         try:
             print('due date:    %s' % issue.due_date)
-        except ResourceAttrError:
+        except rm_exc.ResourceAttrError:
             pass
         try:
             print('parent:      %s' % issue.parent['id'])
-        except ResourceAttrError:
+        except rm_exc.ResourceAttrError:
             pass
         print('updated_on:  %s' % issue.updated_on)
         if hasattr(issue, 'description'):
@@ -135,10 +140,26 @@ def print_project(rmine, proj, verbose=False):
         print('identifier: %s' % proj.identifier)
         try:
             print('parent: %s' % proj.parent['name'])
-        except ResourceAttrError:
+        except rm_exc.ResourceAttrError:
             pass
     print('\n')
 
+
+def editor_text(initial_description=""):
+    EDITOR = os.environ.get('EDITOR')
+    text = initial_description
+    if EDITOR:
+        with tempfile.NamedTemporaryFile(suffix=".tmp", delete=False,
+                                         dir='/tmp/') as tmp:
+            tmp.write(initial_description)
+            tmp.flush()
+            call([EDITOR, tmp.name])
+        with open(tmp.name, 'r') as fh:
+            text = fh.read()
+        os.remove(tmp.name)
+    return text
+
+
 def create_relation(rmine, issue, relissue, reltype):
     """Creates a new issue relationship between two issues."""
 
@@ -147,6 +168,16 @@ def create_relation(rmine, issue, relissue, reltype):
                                 relation_type=reltype)
 
 
+def get_priority(rmine, priority):
+    """Gets the id for the priority passed in."""
+
+    priorities = rmine.enumeration.filter(resource='issue_priorities')
+    p = [p for p in priorities if p.name.lower() == priority.lower()]
+    if len(p) == 0:
+        raise RuntimeError("Priority '%s' is not a priority.")
+    return p[0]
+
+
 def issues(args, rmine):
     """Handle issues"""
 
@@ -166,12 +197,22 @@ def issues(args, rmine):
             qdict['subproject_id'] = '!*'
         if args.assigned_to:
             qdict['assigned_to_id'] = get_user(rmine, args.assigned_to)
+        if args.mine:
+            my_id = rmine.user.get('current').id
+            qdict['assigned_to_id'] = my_id
         if args.status:
             qdict['status_id'] = args.status
         if args.query_id:
+            if not args.project:
+                raise RuntimeError("query_id argument requires '--project "
+                                   "[projectid]' argument also")
             qdict['query_id'] = args.query_id
         # Get the issues
         ishes = rmine.issue.filter(**qdict)
+        if args.priority:
+            priority = get_priority(rmine, args.priority)
+            ishes = [i for i in ishes
+                     if i.priority.name.lower() == priority.name.lower()]
         # This output is kinda lame, but functional for now
         for issue in ishes:
             print_issue(rmine, issue, args.verbose, args.oneline)
@@ -187,9 +228,6 @@ def issues(args, rmine):
             raise RuntimeError('project and subject must be defined')
         idict['project_id'] = args.project
         idict['subject'] = args.subject
-        # Figure out type
-        if not args.type:
-            args.type = 'Bug'
         # Get tracker by type
         itype = [
             tracker.id for tracker in rmine.tracker.all() if
@@ -203,6 +241,8 @@ def issues(args, rmine):
         # Would be rad to do a git commit like editor pop up here
         if args.description:
             idict['description'] = args.description
+        else:
+            idict['description'] = editor_text()
         # figure out the status
         if args.status:
             stat = [status for status in rmine.issue_status.all() if
@@ -211,6 +251,10 @@ def issues(args, rmine):
                 idict['status_id'] = stat[0].id
             except IndexError:
                 raise RuntimeError('Unknown issue type %s' % args.type)
+        # set priority
+        if args.priority:
+            p = get_priority(rmine, args.priority)
+            idict['priority_id'] = p.id
         # Create the issue
         issue = rmine.issue.create(**idict)
         # Create a relationship if one was asked for
@@ -252,6 +296,8 @@ def issues(args, rmine):
             udict['subject'] = args.subject
         if args.description:
             udict['description'] = args.description
+        if args.priority:
+            udict['priority_id'] = get_priority(rmine, args.priority).id
         if args.notes:
             udict['notes'] = args.notes
 
@@ -313,6 +359,7 @@ def issues(args, rmine):
                                                  relation.relation_type,
                                                  relation.issue_to_id))
 
+
 def projects(args, rmine):
     """Handle projects"""
 
@@ -358,14 +405,15 @@ def cmd():
     issues_parser.add_argument('--update', action='store_true',
                                help='Update an existing ticket')
     issues_parser.add_argument('--list-types', action='store_true',
-                               help='List available issue types. Specify a'
-                               'project ID to get specific types for that'
+                               help='List available issue types. Specify a '
+                               'project ID to get specific types for that '
                                'project')
     issues_parser.add_argument('--list-queries', action='store_true',
                                help='List available issue queries')
     issues_parser.add_argument('--list-statuses', action='store_true',
                                help='List available statuses.')
-    issues_parser.add_argument('--delete_relation', help='Delete a relationship',
+    issues_parser.add_argument('--delete_relation',
+                               help='Delete a relationship',
                                type=int, metavar='RELATION_ID')
     # details
     issues_parser.add_argument('--project', help='Filter by or assign to '
@@ -375,9 +423,13 @@ def cmd():
     # I don't like the asterisk here, change it to something else soon
     issues_parser.add_argument('--nosubs', help='Filter out issues from sub '
                                'projects', action='store_true')
+    group = issues_parser.add_mutually_exclusive_group()
     # Need a way to filter all assigned issues, just show unassigned
-    issues_parser.add_argument('--assigned_to', help='Filter by or assign to '
-                               'user. Defaults to UNASSIGNED when creating.')
+    group.add_argument('--assigned_to', help='Filter by or assign to '
+                       'user. Defaults to UNASSIGNED when creating.')
+    group.add_argument('--mine', action='store_true', help='Only your issues')
+    issues_parser.add_argument('--priority', help='Filter by or create '
+                               'priority. Defaults to Normal')
     issues_parser.add_argument('--status',
                                help='Only deal with issues with this status '
                                'or set an issue to this status.')
@@ -387,7 +439,9 @@ def cmd():
                                'creating a new issue')
     issues_parser.add_argument('--notes', help='Notes to use when resolving '
                                'or closing an issue')
-    issues_parser.add_argument('--query_id', help='Filter by query ID')
+    issues_parser.add_argument('--query_id', help='Filter by query ID. '
+                               ' Requires --project [project] and '
+                               '--query arguments.')
     issues_parser.add_argument('--relate_to', help='Create a relationship',
                                type=int)
     issues_parser.add_argument('--relation_type', help='Type of relationship '
@@ -439,28 +493,48 @@ def cmd():
     stderrhandler = logging.StreamHandler()  # Defaults to stderr
     stderrhandler.setLevel(logging.WARNING)
     stderrhandler.setFormatter(formatter)
-    log.addHandler(stdouthandler)
-    log.addHandler(stderrhandler)
+    LOG.addHandler(stdouthandler)
+    LOG.addHandler(stderrhandler)
 
     if args.v:
-        log.setLevel(logging.DEBUG)
+        LOG.setLevel(logging.DEBUG)
     elif args.q:
-        log.setLevel(logging.WARNING)
+        LOG.setLevel(logging.WARNING)
     else:
-        log.setLevel(logging.INFO)
+        LOG.setLevel(logging.INFO)
 
     # load the credentials
     if not args.config:
         args.config = '~/.rore'
     configfile = os.path.expanduser(args.config)
     cparser = ConfigParser.SafeConfigParser()
-    cparser.readfp(open(configfile, 'r'))
+    try:
+        cparser.readfp(open(configfile, 'r'))
+    except IOError:
+        LOG.error("Couldn't find config file: %s" % configfile)
+        exit(1)
 
     siteurl = cparser.get(args.site, 'url')
     key = cparser.get(args.site, 'key')
+    try:
+        verify = cparser.getboolean(args.site, 'verify')
+    except ConfigParser.NoOptionError:
+        verify = False
+
+    if not args.type:
+        try:
+            args.type = cparser.get(args.site, 'default issue tracker')
+        except ConfigParser.NoOptionError:
+            args.type = 'Bug'
+
+    if not args.project:
+        try:
+            args.type = cparser.get(args.site, 'default issue project')
+        except ConfigParser.NoOptionError:
+            pass
 
     # Figure out a way to make this a config option in .rore
-    rmine = redmine.Redmine(siteurl, key=key, requests={'verify': False})
+    rmine = redmine.Redmine(siteurl, key=key, requests={'verify': verify})
 
     # Run the required command -- pass args into it for reference
     args.command(args, rmine)
